package tournament.cup;

import java.awt.*;
import java.awt.event.*;
import java.util.ResourceBundle;
import javax.swing.*;

public class CupField extends JPanel implements MouseListener {
  
  private static final Color DARK_GREEN = new Color(0x009900);
  public static final Dimension PREFERRED_SIZE = new Dimension(100,35);
  
  public static final int FINAL = 0x0001;
  public static final int SEMI = 0x0002;
  public static final int QUARTER =0x0004;
  public static final int SEMIQUARTER = 0x0008;
  public static final int NORMAL = 0x0010;
  public static final int FIRST_ROUND = 0x0020;
  
  private JLabel matchLabel;
  private PlayerLabel playerLabel1, playerLabel2;
  
  private boolean announced = false;
  private ResourceBundle bundle = ResourceBundle.getBundle("Cup");
  private Cup cup;
  private boolean freeRound = false; //Does a player go without a match if the opponent is missing. Always true for the first round and later if a player has been disqualified.
  private int matchNumber;
  private int modifiers;
  private boolean upperStarts = true;
  
  public CupField(Cup _cup, int _modifiers,int _matchNumber, boolean _upperStarts) {
    cup = _cup;
    matchNumber = _matchNumber;
    modifiers = _modifiers;
    upperStarts = _upperStarts;
    
    setBackground(Color.white);
    
    matchLabel = new JLabel("" + matchNumber + " ");
    matchLabel.addMouseListener(this);
    matchLabel.setFont(new Font("Dialog", Font.BOLD, 12));
    playerLabel1 = new PlayerLabel(this);
    playerLabel2 = new PlayerLabel(this);
    
    JPanel panel = new JPanel(new BorderLayout());
    panel.setOpaque(false);
    
    if((modifiers & FINAL) != 0) {
      JPanel upPanel = new JPanel(new BorderLayout());
      upPanel.setOpaque(false);
      upPanel.add(new JLabel(new ImageIcon(ClassLoader.getSystemResource("images/shadow.gif"))), BorderLayout.WEST);
      upPanel.add(playerLabel1, BorderLayout.CENTER);
      JPanel downPanel = new JPanel(new BorderLayout());
      downPanel.setOpaque(false);
      downPanel.add(new JLabel(new ImageIcon(ClassLoader.getSystemResource("images/shadow.gif"))), BorderLayout.WEST);
      downPanel.add(playerLabel2, BorderLayout.CENTER);
      panel.add(upPanel, BorderLayout.NORTH);
      panel.add(downPanel, BorderLayout.SOUTH);
    } else {
      panel.add(playerLabel1, BorderLayout.NORTH);
      panel.add(playerLabel2, BorderLayout.SOUTH);
    }
    
    setLayout(new BorderLayout());
    add(matchLabel, BorderLayout.WEST);
    add(panel, BorderLayout.CENTER);
    
    setPreferredSize(PREFERRED_SIZE);
    setEnabled(false);
    
    if(isFirstRound()) {
      setFreeRound(true);
    }
    
    update();
  }
  
  public String getLoser() {
    return !playerLabel1.winner?playerLabel1.getText():playerLabel2.getText();
  }
  
  public String getName() {
    if(playerLabel1.isEmpty() && playerLabel2.isEmpty()) return "Empty";
    else if(!playerLabel1.isEmpty() && !playerLabel2.isEmpty()) {
      String kamp = bundle.getString("kamp");
      kamp = kamp.replaceAll("<game_id>", "" + matchNumber);
      kamp = kamp.replaceAll("<player1>", playerLabel1.getText());
      kamp = kamp.replaceAll("<player2>", playerLabel2.getText());
      return kamp;
    } else {
      return bundle.getString("advanceWithoutMatch").replaceAll("<name>", playerLabel1.isEmpty()?playerLabel2.getText():playerLabel1.getText());
    }
  }
  
  public String getWinner() {
    return playerLabel1.winner?playerLabel1.getText():playerLabel2.getText();
  }
  
  public boolean hasPlayer(String player) {
    return (!playerLabel1.isEmpty() && playerLabel1.getText().equals(player)) || (!playerLabel2.isEmpty() && playerLabel2.getText().equals(player));
  }
  
  public boolean isEmpty(boolean upper) {
    return upper?playerLabel1.isEmpty():playerLabel2.isEmpty();
  }
  
  public boolean isEnabled() {
    return !playerLabel1.isEmpty() || !playerLabel2.isEmpty();
  }
  
  public boolean isFinished() {
    return playerLabel1.winner || playerLabel2.winner;
  }
  
  public boolean isFirstRound() {
    return (modifiers & FIRST_ROUND) != 0;
  }
  
  public boolean isFreeRound() {
    return freeRound;
  }
  
  public boolean isUnannounced() {
    return !announced;
  }
  
  public void mouseClicked(MouseEvent e) {
    if(!isEnabled()) return;
    Object src = e.getSource();
    
    boolean leftClick = e.getButton() == MouseEvent.BUTTON1;
    
    if(src == playerLabel1 || src == playerLabel2) {
      if(!leftClick) {
        if(src == playerLabel1) {
          playerLabel1.showPopup(e);
        } else {
          playerLabel2.showPopup(e);
        }
        return;
      }
      
      String text = bundle.getString("congratulations");
      text = text.replaceAll("<name>", src == playerLabel1?playerLabel1.getText():playerLabel2.getText());
      String round;
      
      if((modifiers & FINAL) != 0) {
        text = "";
      } else if((modifiers & SEMI) != 0) {
        text = text.replaceAll("<round>", bundle.getString("roundFinal"));
      } else if((modifiers & QUARTER) != 0) {
        text = text.replaceAll("<round>", bundle.getString("roundSemi"));
      } else if((modifiers & SEMIQUARTER) != 0) {
        text = text.replaceAll("<round>", bundle.getString("roundQuarter"));
      } else {
        text = text.replaceAll("<round>", bundle.getString("roundNormal"));
      }
      
      setWinner(src == playerLabel1);
      cup.setClipboardText(text);
    }  else if(!playerLabel1.isEmpty() || !playerLabel2.isEmpty()) {
      cup.setClipboardText(getName());
      setAnnounced(true);
    }
    
    cup.update();
  }
  
  public void mouseEntered(MouseEvent e) {
    Object src = e.getSource();
    
    if((src == playerLabel1 || src == playerLabel2) && !isFinished()) {
      ((JLabel) src).setForeground(Color.blue);
    } else if(src == matchLabel){
      matchLabel.setForeground(Color.MAGENTA);
    }
  }
  public void mouseExited(MouseEvent e) {
    Object src = e.getSource();
    
    if((src == playerLabel1 || src == playerLabel2) && !isFinished()) {
      update();
    } else if(src == matchLabel){
      matchLabel.setForeground(Color.BLACK);
    }
  }
  public void mousePressed(MouseEvent e) {}
  public void mouseReleased(MouseEvent e) {}
  
  public void setAnnounced(boolean b) {
    announced = b;
    update();
  }
  
  public void setEnabled(boolean b) {
    playerLabel1.setEnabled(b);
    playerLabel2.setEnabled(b);
    matchLabel.setEnabled(b);
    setBorder(BorderFactory.createLineBorder(b?Color.black:Color.gray));
  }
  
  public void setFreeRound(boolean b) {
    freeRound = b;
  }
  
  public void setPlayer(String name, boolean upper) {
    if(!upper ^ upperStarts) {
      playerLabel1.setText(name);
      playerLabel1.initTexts();
    } else {
      playerLabel2.setText(name);
      playerLabel2.initTexts();
    }
    setEnabled(true);
  }
  
  public void setWinner(boolean upper) {
    announced=true;
    playerLabel1.winner = upper;
    playerLabel2.winner = !upper;
    
    if((modifiers & FINAL) != 0) {
      JPanel goldPanel = new JPanel(new BorderLayout());
      goldPanel.setOpaque(false);
      goldPanel.add(playerLabel1.winner?playerLabel1:playerLabel2, BorderLayout.CENTER);
      goldPanel.add(new JLabel(new ImageIcon(ClassLoader.getSystemResource("images/gold.gif"))), BorderLayout.WEST);
      JPanel silverPanel = new JPanel(new BorderLayout());
      silverPanel.setOpaque(false);
      silverPanel.add(playerLabel1.winner?playerLabel2:playerLabel1, BorderLayout.CENTER);
      silverPanel.add(new JLabel(new ImageIcon(ClassLoader.getSystemResource("images/silver.gif"))), BorderLayout.WEST);
      
      JPanel panel = new JPanel(new BorderLayout());
      panel.setOpaque(false);
      panel.add(playerLabel1.winner?goldPanel:silverPanel, BorderLayout.NORTH);
      panel.add(playerLabel1.winner?silverPanel:goldPanel, BorderLayout.SOUTH);
      removeAll();
      add(matchLabel, BorderLayout.WEST);
      add(panel, BorderLayout.CENTER);
      revalidate();
    }
  }
  
  public void update() {
    if(isFinished()) {
      playerLabel1.setForeground(playerLabel1.winner?DARK_GREEN:Color.RED);
      playerLabel2.setForeground(playerLabel2.winner?DARK_GREEN:Color.RED);
    } else {
      playerLabel1.setForeground(announced?Color.BLACK:Color.gray);
      playerLabel2.setForeground(announced?Color.BLACK:Color.gray);
    }
  }
  
  private class PlayerLabel extends JLabel implements ActionListener {
    
    private JPopupMenu popupMenu;
    private JMenuItem renameMenuItem, seenMenuItem;
    
    private String name = null;
    private boolean winner = false;
    
    private PlayerLabel(MouseListener mouseListener) {
      popupMenu = new JPopupMenu();
      renameMenuItem = new JMenuItem();
      renameMenuItem.setEnabled(isFirstRound());
      seenMenuItem =new JMenuItem();
      popupMenu.add(renameMenuItem);
      popupMenu.add(seenMenuItem);
      renameMenuItem.addActionListener(this);
      seenMenuItem.addActionListener(this);
      addMouseListener(mouseListener);
    }
    
    public void actionPerformed(ActionEvent e) {
      if(e.getSource() == renameMenuItem) {
        String str = JOptionPane.showInputDialog(getTrans("newName"), name);
        
        while(str != null && (cup.containsName(str.trim()) && !str.trim().equals(name))) {
          str = JOptionPane.showInputDialog(getTrans("newNameAgain").replaceAll("<new_name>", str), str);
        }
        
        if(str != null) {
          setText(str);
          initTexts();
          cup.update();
        }
      } else if(e.getSource() == seenMenuItem) {
        cup.setClipboardText("/seen " + getText());
      }
    }
    
    public String getText() {
      return name == null ? "" : super.getText();
    }
    
    private String getTrans(String text) {
      return bundle.getString(text).replaceFirst("<name>", getText());
    }
    
    private void initTexts() {
      renameMenuItem.setText(getTrans("renameMenuItem"));
      seenMenuItem.setText(getTrans("seenMenuItem"));
    }
    
    private boolean isEmpty() {
      return name == null;
    }
    
    public void setText(String str) {
      name = str;
      super.setText(str);
    }
    
    private void showPopup(MouseEvent e) {
      popupMenu.show(this, e.getX(), e.getY());
    }
  }
}
