package tournament.div;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

import tournament.div.*;

public class Player
        extends JPanel implements ActionListener {
  
  public enum Merit {
    GOLD, SILVER, WINNER, NONE
  }
  
  private static final Color DARK_GREEN = new Color(0x00bb00);
  private static final Color GOLD = new Color(0xbbbb00);
  private static final Color SILVER = new Color(0x666666);
  
  //Strings, can be translated with properties-files
  /*private String advancePlayer, cancelPlayer, confirmDisqualificationMessage,
          disqualify, confirmDisqualificationTitle, giveGold, giveSilver, grat;*/
  
  private JMenuItem advanceMenuItem, disqualifyMenuItem, goldMenuItem, renameMenuItem, seenMenuItem, silverMenuItem;
  private JPopupMenu popupMenu;
  private JTextField nameField, pointsField, gamesField;
  
  private ResourceBundle bundle = ResourceBundle.getBundle("Division");
  private boolean disqualified = false; //Has the player been disqualified
  private Division division; //Division which the player belongs to
  private int games = 0;
  private int lotteryChances; //The smaller value, the better chances
  private ArrayList<Match> matches = new ArrayList<Match> ();
  private Merit merit = Merit.NONE;
  private String name; //Player's name
  private float points = 0;
  
  public Player(Division _division, String _name, int _lotteryChances) {
    division = _division;
    name = _name;
    lotteryChances = _lotteryChances;
    
    initComponents();
    initTexts();
    update();
  }
  
  public void actionPerformed(ActionEvent e) {
    Object src = e.getSource();
    
    if (src == disqualifyMenuItem) {
      if(!disqualified) {
        int sel = JOptionPane.showConfirmDialog(division,
                getTrans("confirmDisqualificationMessage"),
                getTrans("confirmDisqualificationTitle"),
                JOptionPane.YES_NO_OPTION);
        setDisqualified(sel == JOptionPane.YES_OPTION);
      } else {
        setDisqualified(false);
      }
    } else if (src == goldMenuItem) {
      setMerit(merit == Merit.GOLD ? Merit.NONE : Merit.GOLD);
    }else if(src == renameMenuItem) {
      String str = JOptionPane.showInputDialog(getTrans("newName"), name);
      
      while(str != null && (division.getLeague().containsName(str.trim()) && !str.trim().equals(name))) {
        str = JOptionPane.showInputDialog(getTrans("newNameAgain").replaceAll("<new_name>", str), str);
      }
      
      if(str != null) {
        name = str;
        initTexts();
        division.updateAll();
      }
      
    } else if(src == seenMenuItem) {
      division.getLeague().setClipboardText("/seen " + name);
    } else if (src == silverMenuItem) {
      setMerit(merit == Merit.SILVER ? Merit.NONE : Merit.SILVER);
    } else if (src == advanceMenuItem) {
      setMerit(merit == Merit.WINNER ? Merit.NONE : Merit.WINNER);
    }
    
    update();
  }
  
  public void addMatch(Match match) {
    matches.add(match);
  }
  
  public Division getDivision() {
    return division;
  }
  
  public int getGames() {
    return games;
  }
  
  public int getLotteryChances() {
    return lotteryChances;
  }
  
  public int getLosses() {
    int c = 0;
    for (Match m : matches) {
      if (m.isLossFor(this)) {
        c++;
      }
    }
    return c;
  }
  
  public Merit getMerit() {
    return merit;
  }
  
  public String getName() {
    return name;
  }
  
  public float getPoints() {
    return points;
  }
  
  private String getTrans(String text) {
    return bundle.getString(text).replaceAll("<name>", name);
  }
  
  //Check if this player has played against some other player
  public boolean hasPlayed(Player p) {
    for (Match m : matches) {
      if (m.hasPlayer(p)) {
        return true;
      }
    }
    return false;
  }
  
  private void initComponents() {
    setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));
    
    Font font = new Font("Dialog", Font.BOLD, 12);
    
    /****nameField****/
    nameField = new JTextField();
    nameField.setBackground(Color.white);
    nameField.setEditable(false);
    nameField.setFont(font);
    Dimension dim = new Dimension(180, 20);
    nameField.setPreferredSize(dim);
    add(nameField);
    
    /****gamesField****/
    gamesField = new JTextField();
    gamesField.setBackground(Color.white);
    gamesField.setEditable(false);
    gamesField.setFont(font);
    gamesField.setHorizontalAlignment(JTextField.RIGHT);
    dim = new Dimension(40, 20);
    gamesField.setPreferredSize(dim);
    gamesField.setMaximumSize(dim);
    add(gamesField);
    
    /****pointsField****/
    pointsField = new JTextField();
    pointsField.setBackground(Color.white);
    pointsField.setEditable(false);
    pointsField.setFont(font);
    pointsField.setHorizontalAlignment(JTextField.RIGHT);
    dim = new Dimension(40, 20);
    pointsField.setPreferredSize(dim);
    pointsField.setMaximumSize(dim);
    add(pointsField);
    
    /****Popup****/
    popupMenu = new JPopupMenu();
    
    advanceMenuItem = new JMenuItem();
    disqualifyMenuItem = new JMenuItem();
    disqualifyMenuItem.setForeground(Color.red);
    goldMenuItem = new JMenuItem();
    goldMenuItem.setForeground(GOLD);
    renameMenuItem = new JMenuItem();
    seenMenuItem = new JMenuItem();
    silverMenuItem = new JMenuItem();
    silverMenuItem.setForeground(SILVER);
    
    
    if (division.isFinal()) {
      popupMenu.add(goldMenuItem);
      popupMenu.add(silverMenuItem);
    } else {
      popupMenu.add(advanceMenuItem);
    }
    
    popupMenu.add(seenMenuItem);
    popupMenu.add(renameMenuItem);
    popupMenu.addSeparator();
    popupMenu.add(disqualifyMenuItem);
    
    advanceMenuItem.addActionListener(this);
    disqualifyMenuItem.addActionListener(this);
    goldMenuItem.addActionListener(this);
    renameMenuItem.addActionListener(this);
    seenMenuItem.addActionListener(this);
    silverMenuItem.addActionListener(this);
    
    nameField.addMouseListener(new gui.PopupListener(popupMenu));
  }
  
  //This method is separate of initComponents because the texts may have to be changed if the player's name is renamed
  private void initTexts() {
    advanceMenuItem.setText(getTrans(merit==Merit.WINNER?"cancelMenuItem":"advanceMenuItem"));
    disqualifyMenuItem.setText(getTrans(disqualified?"cancelMenuItem":"disqualifyMenuItem"));
    gamesField.setToolTipText(getTrans("playedGamesToolTip"));
    goldMenuItem.setText(getTrans(merit==Merit.GOLD?"cancelMenuItem":"goldMenuItem"));
    nameField.setText(name);
    nameField.setToolTipText(getTrans("nameToolTip"));
    pointsField.setToolTipText(getTrans("pointsToolTip"));
    renameMenuItem.setText(getTrans("renameMenuItem"));
    seenMenuItem.setText(getTrans("seenMenuItem"));
    silverMenuItem.setText(getTrans(merit==Merit.SILVER?"cancelMenuItem":"silverMenuItem"));
  }
  
  public boolean isDisqualified() {
    return disqualified;
  }
  
  public boolean isPlaying() {
    for (Match m : matches) {
      if (!m.isFinished() && !m.isCancelled()) {
        return true;
      }
    }
    return false;
  }
  
  public void setDisqualified(boolean b) {
    disqualified = b;
    initTexts();
    division.updateAll();
  }
  
  public void setEnabled(boolean b) {
    advanceMenuItem.setEnabled(b);
    disqualifyMenuItem.setEnabled(b);
    goldMenuItem.setEnabled(b);
    renameMenuItem.setEnabled(b);
    seenMenuItem.setEnabled(b);
    silverMenuItem.setEnabled(b);
  }
  
  public void setMerit(Merit m) {
    merit = m;
    initTexts();
    if (merit == Merit.WINNER) {
      division.getLeague().setClipboardText(getTrans("congratulations"));
    }
    division.updateAll();
  }
  
  public void update() {
    points = games = 0;
    
    if (disqualified) {
      nameField.setForeground(Color.red);
      gamesField.setForeground(Color.red);
      pointsField.setForeground(Color.red);
      
      gamesField.setText("x");
      pointsField.setText("x");
      return;
    }
    
    Color foreground;
    if(merit == Merit.GOLD) foreground = GOLD;
    else if(merit == Merit.SILVER) foreground = SILVER;
    else if(merit == Merit.WINNER) foreground = DARK_GREEN;
    else foreground = Color.black;
    
    nameField.setForeground(foreground);
    gamesField.setForeground(foreground);
    pointsField.setForeground(foreground);
    
    for (Match m : matches) {
      if (m.isFinished() && !m.isCancelled()) {
        points += m.getPointsFor(this);
        games++;
      }
    }
    
    nameField.setText(name);
    gamesField.setText(Integer.toString(games));
    pointsField.setText(Integer.toString( (int) points) +
            (points > (int) points ? '' : ""));
  }
}
